@@ -3,23 +3,27 @@ from typing import NamedTuple
 
 from .builders import EntryBuilder, ContainerBuilder
 from .muid import Muid
-from struct import pack
+from struct import pack, unpack
 from .typedefs import Medallion, MuTimestamp
 
 
 class Chain(NamedTuple):
     """ Pair of numbers to identify a blockchain in gink. """
-    chain_start: MuTimestamp
     medallion: Medallion
+    chain_start: MuTimestamp
 
     def __bytes__(self):
-        return pack(">QQ", self.chain_start, self.medallion)
+        return pack(">QQ", self.medallion, self.chain_start)
+
+    @staticmethod
+    def from_bytes(data: bytes) -> 'Chain':
+        return Chain(*unpack(">QQ", data))
 
 
 class FoundEntry(NamedTuple):
     """ Entry information returned by the store for keyed containers.
 
-    The address is necessary because some muids in the proto may be relative.
+        The address is necessary because some muids in the proto may be relative.
     """
     address: Muid
     builder: EntryBuilder
@@ -34,14 +38,14 @@ class FoundEntry(NamedTuple):
         return hash(self.address)
 
 
-
 class FoundContainer(NamedTuple):
+    """ Muid and Builder for a container. """
     address: Muid
     builder: ContainerBuilder
 
 
 class PositionedEntry(NamedTuple):
-    """ represents data in a positioned data structure like Sequence """
+    """ Represents data in a positioned data structure like Sequence """
     position: MuTimestamp
     positioner: Muid
     entry_muid: Muid
